/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.office.json.osgi;

import com.openexchange.ajax.requesthandler.osgiservice.AJAXModuleActivator;
import com.openexchange.config.ConfigurationService;
import com.openexchange.documentconverter.IManager;
import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
import com.openexchange.filemanagement.DistributedFileManagement;
import com.openexchange.filemanagement.ManagedFileManagement;
import com.openexchange.folderstorage.FolderService;
import com.openexchange.office.IResourceProvider;
import com.openexchange.office.json.actions.DocumentFilterActionFactory;
import com.openexchange.preview.PreviewService;
import com.openexchange.realtime.dispatch.MessageDispatcher;

/**
 * {@link DocumentFilterActivator}
 *
 * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
 */

public class DocumentFilterActivator extends AJAXModuleActivator {


    private DocumentFilterActionFactory actionFactory = null;

    /**
     * Initializes a new {@link DocumentFilterActivator}.
     */
    public DocumentFilterActivator() {
        super();
    }

    /* (non-Javadoc)
     * @see com.openexchange.osgi.DeferredActivator#getNeededServices()
     */
    @Override
    protected Class<?>[] getNeededServices() {
        return new Class<?>[] {
            com.openexchange.office.odf.Importer.class,
            com.openexchange.office.odf.Exporter.class,
            com.openexchange.office.ooxml.pptx.Importer.class,
            com.openexchange.office.ooxml.pptx.Exporter.class,
            com.openexchange.office.ooxml.docx.Importer.class,
            com.openexchange.office.ooxml.docx.Exporter.class,
            com.openexchange.office.ooxml.xlsx.Importer.class,
            com.openexchange.office.ooxml.xlsx.Exporter.class,
            IResourceProvider.class,
            ConfigurationService.class
        };
    }

    /* (non-Javadoc)
     * @see com.openexchange.osgi.HousekeepingActivator#getOptionalServices()
     */
    @Override
    protected Class<?>[] getOptionalServices() {
        return new Class<?>[] {
            DistributedFileManagement.class,
            FolderService.class,
            IDBasedFileAccessFactory.class,
            IManager.class,
            ManagedFileManagement.class,
            MessageDispatcher.class,
            PreviewService.class
        };
    }

    /* (non-Javadoc)
     * @see com.openexchange.osgi.DeferredActivator#startBundle()
     */
    @Override
    protected void startBundle() throws Exception {
        openTrackers();

        actionFactory = new DocumentFilterActionFactory(this);
        registerModule(actionFactory, "oxodocumentfilter");
    }

    @Override
    protected void stopBundle() throws Exception {
        ;
    }

}
